@@ -1,7 +1,7 @@
 ;; -*- mode: emacs-lisp -*-
 ;; mew-shimbun.el --- View shimbun contents with Mew
 
-;; Copyright (C) 2001 TSUCHIYA Masatoshi <tsuchiya@namazu.org>
+;; Copyright (C) 2001, 2002, 2003 TSUCHIYA Masatoshi <tsuchiya@namazu.org>
 
 ;; Author: TSUCHIYA Masatoshi <tsuchiya@namazu.org>
 ;;         Hideyuki SHIRAI <shirai@meadowy.org>
@@ -25,29 +25,11 @@
 ;; Inc.; 59 Temple Place, Suite 330; Boston, MA 02111-1307, USA.
 
 ;;; Commentary:
-;; This package is SHIMBUN interface for Mew version 2.1 or later.
+;; This package is `Shimbun' interface for Mew version 2.1 or later.
+;; SHIMBUN 
 
-;;; Instalation:
-;; Simply load this file and add followings in your ~/.mew file.
-;; If you can understand Japanese, please read 'README.shimbun.ja'.
-;;
-;;; Comment out below one line, if you use 'Mew Shimbun unseen mark'.
-;;
-;;; (setq mew-shimbun-use-unseen t)
-;;;; (setq mew-shimbun-use-unseen-cache-save t)
-;;
-;; (setq mew-shimbun-folder-groups .... ;; at your service
-;;
-;; (require 'mew-shimbun)
-;; (define-key mew-summary-mode-map "G"  (make-sparse-keymap))
-;; (define-key mew-summary-mode-map "Gg" 'mew-shimbun-goto-folder)
-;; (define-key mew-summary-mode-map "GG" 'mew-shimbun-goto-unseen-folder)
-;; (define-key mew-summary-mode-map "Gi" 'mew-shimbun-retrieve)
-;; (define-key mew-summary-mode-map "GI" 'mew-shimbun-retrieve-all)
-;; (define-key mew-summary-mode-map "Gr" 'mew-shimbun-re-retrieve)
-;; (define-key mew-summary-mode-map "GR" 'mew-shimbun-re-retrieve-all)
-;; (define-key mew-summary-mode-map "Ge" 'mew-shimbun-expire)
-;; (define-key mew-summary-mode-map "GE" 'mew-shimbun-expire-all)
+;;; Instalation & Usage:
+;; Please read the emacs-w3m info (C-h i m emacs-w3m(-ja) RET m Mew Shimbun RET).
 ;;
 
 ;;; Code:
@@ -89,7 +71,7 @@
   "*The folder where SHIMBUN are contained."
   :group 'shimbun
   :group 'mew-shimbun
-  :type 'string)
+  :type '(string :size 0))
 
 (defcustom mew-shimbun-folder-groups nil
   "*Alist of `shimbun folder name (exclude `mew-shimbun-folder')'
@@ -112,19 +94,24 @@ show below example,
   :group 'shimbun
   :group 'mew-shimbun
   :type '(repeat
-	  (cons (string :tag "Folder")
-		(repeat
-		 (cons (string :tag "Server.Group")
-		       (choice :tag "Index checking range"
-			       (const all)
-			       (const last)
-			       (integer :tag "pages")))))))
+	  (cons
+	   :format "%v" :indent 2
+	   (string :format "Folder: %v\n" :size 0)
+	   (repeat
+	    :format "%{Server.Group + Index_Checking_Range%}:\n %v%i\n"
+	    :indent 3 :sample-face underline
+	    (cons :format "%v" :indent 6
+		  (string :format "Server.Group: %v\n" :size 0)
+		  (radio :format "Range: %v " :value all
+			 (const :format "%v " all)
+			 (const :format "%v " last)
+			 (integer :format "Pages: %v\n" :size 0)))))))
 
 (defcustom mew-shimbun-db-file ".mew-shimbun-db"
   "*File name of mew-shimbun database."
   :group 'shimbun
   :group 'mew-shimbun
-  :type 'file)
+  :type '(file :size 0))
 
 (defcustom mew-shimbun-expires nil
   "*Alist of `shimbun folder name' and expire days.
@@ -136,8 +123,9 @@ Show below expire,
   :group 'shimbun
   :group 'mew-shimbun
   :type '(repeat
-	  (cons (string :tag "Folder")
-		(integer :tag "Days"))))
+	  (cons :format "%v" :indent 11
+		(string :format "Folder: %v\n" :size 0)
+		(integer :format "Days: %v\n" :size 0))))
 
 (defcustom mew-shimbun-use-expire-pack nil
   "*If non-nin, exec `pack' after expire."
@@ -160,21 +148,23 @@ show below example,
 "
   :group 'shimbun
   :group 'mew-shimbun
-  :type '(choice
-	  (const :tag "same 'mew-lisp-max-length'" nil)
-	  (integer :tag "limit for all group" :value 2000)
-	  (repeat :tag "alist folder and length"
-		  (cons (choice :tag "Folder"
-				(string :tag "folder")
-				(const :tag "other" t))
-			(choice :tag "Max length of database"
-				(integer :tag "length" :value 2000))))))
+  :type '(radio
+	  (const :tag "Same as `mew-lisp-max-length'" nil)
+	  (integer :format "Limit for all groups: %v\n"
+		   :size 0 :value 2000)
+	  (repeat :indent 4 :tag "Alist of folders and lengths"
+		  (cons :format "%v" :indent 8
+			(radio :format "%v" :value t
+			       (const :format "Other " t)
+			       (string :format "Folder: %v\n" :size 0))
+			(integer :format "Maximum length of database: %v\n"
+				 :size 0 :value 2000)))))
 
 (defcustom mew-shimbun-unknown-from "foo@bar.baz"
   "*Shimbun mail address when From header is strange."
   :group 'shimbun
   :group 'mew-shimbun
-  :type 'string)
+  :type '(string :size 0))
 
 (defcustom mew-shimbun-mark-re-retrieve mew-mark-multi
   "*Shimbun re-retrieve mark."
@@ -218,13 +208,6 @@ show below example,
 
 (defvar mew-shimbun-unseen-regex nil)
 
-(defmacro mew-shimbun-unseen-regex ()
-  `(or mew-shimbun-unseen-regex
-       (setq mew-shimbun-unseen-regex
-	     (concat mew-regex-msg "\\("
-		     (regexp-quote (string mew-shimbun-mark-unseen))
-		     "\\)"))))
-
 (defvar mew-shimbun-folder-regex
   (mew-folder-regex (file-name-as-directory mew-shimbun-folder)))
 
@@ -233,6 +216,15 @@ show below example,
 (defvar mew-shimbun-input-hist nil)
 
 ;;; Macro:
+(eval-when-compile
+  (if (fboundp 'static-if)
+      nil
+    (defmacro static-if (cond then &rest else)
+      ;; Like `if', but evaluate COND at compile time.
+      (if (eval cond)
+	  then
+	`(progn ,@else)))))
+
 (defmacro mew-shimbun-db-search-id (id)
   `(assoc ,id mew-shimbun-db))
 
@@ -257,17 +249,68 @@ show below example,
 
 (put 'mew-shimbun-element-body 'lisp-indent-function 1)
 
+(defmacro mew-shimbun-headers (shimbun range)
+  `(let ((w3m-process-wait-discard-input t))
+     (shimbun-headers ,shimbun ,range)))
+
+(defmacro mew-shimbun-article (shimbun head)
+  `(let ((w3m-process-wait-discard-input t))
+     (shimbun-article ,shimbun ,head)))
+
 (defsubst mew-shimbun-mode-display (group server get count sum)
   (mew-summary-lock 'shimbun
 		    (format mew-shimbun-lock-format2 group server get count sum))
   (force-mode-line-update))
 
-(w3m-static-if (fboundp 'mew-summary-visit-folder)
+(static-if (fboundp 'mew-summary-visit-folder)
     (defalias 'mew-shimbun-visit-folder 'mew-summary-visit-folder)
   (defun mew-shimbun-visit-folder (folder)
     (mew-summary-ls
      (mew-summary-switch-to-folder folder))))
 
+(defun mew-shimbun-unseen-regex ()
+  (static-if (boundp 'mew-regex-msg)
+      ;; Mew3
+      (setq mew-shimbun-unseen-regex
+	    (concat mew-regex-msg (regexp-quote (string mew-shimbun-mark-unseen))))
+    ;; Mew4
+    (setq mew-shimbun-unseen-regex
+	  (concat "^" (regexp-quote (string mew-shimbun-mark-unseen))))))
+  
+(defun mew-shimbun-set-form (fld)
+  (static-if (fboundp 'mew-summary-scan-form)
+      ;; Mew3
+      (unless (mew-sinfo-get-scan-form)
+	(mew-sinfo-set-scan-form (mew-summary-scan-form fld)))
+    ;; Mew4
+    (unless (mew-sinfo-get-summary-form)
+      (mew-sinfo-set-summary-form (mew-get-summary-form fld)))))
+
+(if (featurep 'xemacs)
+    nil
+  (eval-and-compile
+    (autoload 'ad-arglist "advice"))
+  (eval-when-compile
+    (defmacro function-max-args (function)
+      ;; Return the maximum number of arguments a function may be called with.
+      ;; The function may be any form that can be passed to `funcall',
+      ;; any special form, or any macro.
+      ;; If the function takes an arbitrary number of arguments or is
+      ;; a built-in special form, nil is returned."
+      (let ((fn (make-symbol "emulating-function-max-args-function"))
+	    (arglist (make-symbol "emulating-function-max-args-arglist")))
+	`(let* ((,fn ,function)
+		(,arglist (ad-arglist (progn
+					(while (symbolp ,fn)
+					  (setq ,fn (symbol-function ,fn)))
+					,fn))))
+	   (cond ((memq '&rest ,arglist)
+		  nil)
+		 ((memq '&optional ,arglist)
+		  (1- (length ,arglist)))
+		 (t
+		  (length ,arglist))))))))
+
 ;;; Main:
 ;;;###autoload
 (defun mew-shimbun-goto-unseen-folder ()
@@ -298,7 +341,8 @@ If called with '\\[universal-argument]', goto folder to have a few new messages.
 	      (if (get-buffer fld)
 		  (with-current-buffer fld
 		    (goto-char (point-min))
-		    (when (re-search-forward (mew-shimbun-unseen-regex) nil t)
+		    (when (re-search-forward (or mew-shimbun-unseen-regex
+						 (mew-shimbun-unseen-regex)) nil t)
 		      (setq sbflds (cons fld sbflds))))
 		(setq cfile (mew-expand-folder fld mew-summary-cache-file))
 		(when (file-readable-p cfile)
@@ -307,7 +351,8 @@ If called with '\\[universal-argument]', goto folder to have a few new messages.
 		     mew-cs-text-for-read mew-cs-dummy
 		     (insert-file-contents cfile nil)
 		     (goto-char (point-min))
-		     (when (re-search-forward (mew-shimbun-unseen-regex) nil t)
+		     (when (re-search-forward (or mew-shimbun-unseen-regex
+						  (mew-shimbun-unseen-regex)) nil t)
 		       (setq sbflds (cons fld sbflds))))))))))))
     (mapcar (lambda (x)
 	      (setq alst (cons (list x) alst)))
@@ -325,7 +370,7 @@ If called with '\\[universal-argument]', goto folder to have a few new messages.
       (setcar mew-shimbun-input-hist fld))
     (setq mew-input-folder-hist (cons fld mew-input-folder-hist))
     (let ((newfld (mew-summary-switch-to-folder fld)))
-      (if (eq 1 (w3m-function-max-args 'mew-summary-ls))
+      (if (eq 1 (function-max-args 'mew-summary-ls))
 	  (mew-summary-ls newfld)
 	(dont-compile;; To avoid a byte-compile warnning.
 	  (mew-summary-ls newfld newfld))))))
@@ -336,7 +381,7 @@ If called with '\\[universal-argument]', goto folder to have a few new messages.
   (interactive)
   (when (mew-summary-exclusive-p)
     (mew-summary-only
-     (let ((fld (mew-summary-folder-name))
+     (let ((fld (mew-summary-folder-name 'ext))
 	   (mua (luna-make-entity 'shimbun-mew-mua))
 	   (count 0)
 	   alst server group range)
@@ -349,8 +394,7 @@ If called with '\\[universal-argument]', goto folder to have a few new messages.
 	   (mew-window-configure 'summary)
 	   (mew-current-set nil nil nil)
 	   (mew-decode-syntax-delete)
-	   (unless (mew-sinfo-get-scan-form)
-	     (mew-sinfo-set-scan-form (mew-summary-scan-form fld)))
+	   (mew-shimbun-set-form fld)
 	   (save-excursion
 	     (dolist (sgr (cdr alst))
 	       (mew-shimbun-element-body sgr group server
@@ -372,7 +416,7 @@ If called with '\\[universal-argument]', goto folder to have a few new messages.
   (interactive)
   (mew-summary-only
    (let ((mua (luna-make-entity 'shimbun-mew-mua))
-	 (cfld (mew-summary-folder-name))
+	 (cfld (mew-summary-folder-name 'ext))
 	 fld dir server group range)
      (run-hooks 'mew-shimbun-before-retrieve-hook)
      (mew-window-configure 'summary)
@@ -412,7 +456,7 @@ If called with '\\[universal-argument]', goto folder to have a few new messages.
       (mew-make-directory (mew-expand-folder fld)))
     (mew-shimbun-db-setup fld)
     (unwind-protect
-	(let* ((headers (shimbun-headers shimbun range))
+	(let* ((headers (mew-shimbun-headers shimbun range))
 	       (sum (length headers)))
 	  (setq headers (sort headers
 			      (lambda (x y)
@@ -428,7 +472,7 @@ If called with '\\[universal-argument]', goto folder to have a few new messages.
 		(with-current-buffer buf
 		  (mew-erase-buffer)
 		  (set-buffer-multibyte nil)
-		  (shimbun-article shimbun head)
+		  (mew-shimbun-article shimbun head)
 		  (setq md5 (mew-shimbun-md5))
 		  (when (and (> (buffer-size) 0)
 			     (mew-shimbun-db-add-id id md5))
@@ -449,9 +493,9 @@ If called with '\\[universal-argument]', goto folder to have a few new messages.
 	      (setq dispcount (1+ dispcount))
 	      (mew-shimbun-mode-display group server count dispcount sum))))
       (mew-summary-unlock)
-      (w3m-static-if (fboundp 'mew-folder-insert)
-	  (mew-folder-insert fld)
-	(mew-local-folder-insert fld))
+      (static-if (fboundp 'mew-local-folder-insert)
+	  (mew-local-folder-insert fld)
+	(mew-folder-insert fld))
       (shimbun-close-group shimbun)
       (shimbun-close shimbun)
       (mew-shimbun-db-shutdown fld count))
@@ -464,7 +508,7 @@ If called with '\\[universal-argument]', re-retrieve messages marked with '@'."
   (interactive "P")
   (when (mew-summary-exclusive-p)
     (mew-summary-only
-     (let* ((fld (mew-summary-folder-name))
+     (let* ((fld (mew-summary-folder-name 'ext))
 	    (msgs (list (progn (mew-summary-goto-message)
 			       (mew-summary-message-number))))
 	    (mua (luna-make-entity 'shimbun-mew-mua))
@@ -480,8 +524,7 @@ If called with '\\[universal-argument]', re-retrieve messages marked with '@'."
 	   (mew-window-configure 'summary)
 	   (mew-current-set nil nil nil)
 	   (mew-decode-syntax-delete)
-	   (unless (mew-sinfo-get-scan-form)
-	     (mew-sinfo-set-scan-form (mew-summary-scan-form fld)))
+	   (mew-shimbun-set-form fld)
 	   (when args
 	     (setq msgs (mew-summary-mark-collect mew-shimbun-mark-re-retrieve)))
 	   (if (null msgs)
@@ -511,7 +554,7 @@ If called with '\\[universal-argument]', re-retrieve messages in the region."
   (interactive "P")
   (when (mew-summary-exclusive-p)
     (mew-summary-only
-     (let* ((fld (mew-summary-folder-name))
+     (let* ((fld (mew-summary-folder-name 'ext))
 	    (mua (luna-make-entity 'shimbun-mew-mua))
 	    (begend (cons (point-min) (point-max)))
 	    (newcount 0) (rplcount 0) (same 0)
@@ -534,8 +577,7 @@ If called with '\\[universal-argument]', re-retrieve messages in the region."
 	       (mew-summary-goto-message)
 	       (setq endmsg (mew-summary-message-number))))
 	   (setq id-msgs (mew-shimbun-get-id-msgs 'range fld begmsg endmsg))
-	   (unless (mew-sinfo-get-scan-form)
-	     (mew-sinfo-set-scan-form (mew-summary-scan-form fld)))
+	   (mew-shimbun-set-form fld)
 	   (mew-window-configure 'summary)
 	   (mew-current-set nil nil nil)
 	   (mew-decode-syntax-delete)
@@ -571,7 +613,7 @@ If called with '\\[universal-argument]', re-retrieve messages in the region."
     (shimbun-open-group shimbun group)
     (mew-shimbun-db-setup2 fld id-msgs)
     (unwind-protect
-	(let* ((headers (shimbun-headers shimbun range))
+	(let* ((headers (mew-shimbun-headers shimbun range))
 	       (sum (length headers)))
 	  (setq headers (sort headers
 			      (lambda (x y)
@@ -599,7 +641,7 @@ If called with '\\[universal-argument]', re-retrieve messages in the region."
 		(with-current-buffer buf
 		  (mew-erase-buffer)
 		  (set-buffer-multibyte nil)
-		  (shimbun-article shimbun head)
+		  (mew-shimbun-article shimbun head)
 		  (when (> (buffer-size) 0)
 		    (setq newmd5 (mew-shimbun-md5))
 		    (if (and (stringp oldmd5) (string= oldmd5 newmd5))
@@ -630,7 +672,7 @@ If called with '\\[universal-argument]', re-retrieve messages in the region."
 (defun mew-shimbun-expire-all ()
   "Expire all shimbun folder."
   (interactive)
-  (let ((cfld (mew-summary-folder-name)) fld)
+  (let ((cfld (mew-summary-folder-name 'ext)) fld)
     (dolist (alst mew-shimbun-expires)
       (setq fld (concat (file-name-as-directory mew-shimbun-folder)
 			(car alst)))
@@ -645,13 +687,23 @@ If called with '\\[universal-argument]', re-retrieve messages in the region."
 	  (mew-kill-buffer (current-buffer)))))
     (mew-shimbun-visit-folder cfld)))
 
+(defun mew-shimbun-pick (&rest args)
+  (apply 'call-process
+	 (static-if (boundp 'mew-prog-mewl) mew-prog-mewl mew-prog-mewls)
+	 nil t nil args))
+
+(defun mew-shimbun-jump-msg (msg)
+  (static-if (fboundp 'mew-regex-jmp-msg)
+      (re-search-forward (mew-regex-jmp-msg msg) nil t)
+    (re-search-forward (format "\r  %s " msg) nil t)))
+
 ;;;###autoload
 (defun mew-shimbun-expire ()
   "Expire this shimbun folder."
   (interactive)
   (when (mew-summary-exclusive-p)
     (mew-summary-only
-     (let* ((fld (mew-summary-folder-name))
+     (let* ((fld (mew-summary-folder-name 'ext))
 	    (days (mew-shimbun-expire-day fld))
 	    (i 0)
 	    file msgs msg-alist begmsg endmsg t1)
@@ -673,11 +725,10 @@ If called with '\\[universal-argument]', re-retrieve messages in the region."
 	       (with-temp-buffer
 		 (mew-piolet
 		  mew-cs-text-for-read mew-cs-text-for-write
-		  (call-process mew-prog-mewls nil t nil
-				"-b" mew-mail-path
-				"-d" "Date:"
-				"-s" (format "%s %s-%s"
-					     fld begmsg endmsg))
+		  (mew-shimbun-pick "-b" mew-mail-path
+				    "-d" "Date:"
+				    "-s" (format "%s %s-%s"
+						 fld begmsg endmsg))
 		  (goto-char (point-min))
 		  (while (not (eobp))
 		    (when (looking-at "^\\([1-9][0-9]*\\): *\\([^\n]+\\)$")
@@ -705,7 +756,7 @@ If called with '\\[universal-argument]', re-retrieve messages in the region."
 		 (setq i (1+ i))
 		 (when (zerop (% i 10))
 		   (message "Expire (%s) %d/%d..." fld i t1))
-		 (when (re-search-forward (mew-regex-jmp-msg msg) nil t)
+		 (when (mew-shimbun-jump-msg msg)
 		   (beginning-of-line)
 		   (mew-elet
 		    (delete-region (point)
@@ -720,7 +771,11 @@ If called with '\\[universal-argument]', re-retrieve messages in the region."
 		(set-buffer-modified-p nil))
 	       (when (and mew-shimbun-use-expire-pack
 			  (> t1 0))
-		 (mew-summary-pack-body))
+		 (if (eq 1 (function-max-args 'mew-summary-pack-body))
+		     (dont-compile
+		       (mew-summary-pack-body fld))
+		   (dont-compile
+		     (mew-summary-pack-body))))
 	       (message "Expire (%s) %d/%d...done" fld t1 t1)))))))))
 
 (defun mew-shimbun-expire-day (fld)
@@ -751,10 +806,9 @@ If called with '\\[universal-argument]', re-retrieve messages in the region."
       (with-temp-buffer
 	(mew-piolet
 	 mew-cs-text-for-read mew-cs-text-for-write
-	 (call-process mew-prog-mewls nil t nil
-		       "-b" mew-mail-path
-		       "-d" "X-Shimbun-Id:"
-		       "-s" (format "%s %s-%s" (nth 0 args) (nth 1 args) (nth 2 args))))
+	 (mew-shimbun-pick "-b" mew-mail-path
+			   "-d" "X-Shimbun-Id:"
+			   "-s" (format "%s %s-%s" (nth 0 args) (nth 1 args) (nth 2 args))))
 	(goto-char (point-min))
 	(while (re-search-forward "^\\([1-9][0-9]*\\): \\([^\n]+\\)" nil t)
 	  (setq id-msgs (cons (cons (match-string 2) (match-string 1)) id-msgs))))
@@ -766,7 +820,7 @@ If called with '\\[universal-argument]', re-retrieve messages in the region."
 (defun mew-shimbun-scan-message (fld msg)
   (set-buffer-multibyte t)
   (let ((width (1- (mew-scan-width)))
-	(vec (w3m-static-if (fboundp 'mew-pop-scan-header)
+	(vec (static-if (fboundp 'mew-pop-scan-header)
 		 (mew-pop-scan-header)
 	       (mew-scan-header))))
     (mew-scan-set-folder vec fld)
@@ -777,10 +831,10 @@ If called with '\\[universal-argument]', re-retrieve messages in the region."
       ;; xxxxx more fast
       (with-current-buffer fld
 	(goto-char (point-min))
-	(when (re-search-forward (mew-regex-jmp-msg msg) nil t)
+	(when (mew-shimbun-jump-msg msg)
 	  (mew-mark-put-mark mew-shimbun-mark-unseen))))
     ;; for summary redraw
-    (sit-for 0.1)))
+    (sit-for 0.01)))
 
 (defun mew-shimbun-sanity-convert ()
   (if (re-search-forward mew-eoh nil t)
@@ -931,9 +985,14 @@ If called with '\\[universal-argument]', re-retrieve messages in the region."
 	(mew-buffer-substring (point-min)
 			      (min (point-max) (+ (point-min) 4096))))))))
 
+(defvar mew-shimbun-touch-folder-p
+  (static-if (boundp 'mew-touch-folder-p)
+      mew-touch-folder-p
+    t)) ;; Mew 4
+
 (defun mew-shimbun-folder-new-p (fld)
   (let* ((dir (file-chase-links (mew-expand-folder fld)))
-	 (tdir (if mew-touch-folder-p
+	 (tdir (if mew-shimbun-touch-folder-p
 		   (mew-file-get-time
 		    (expand-file-name mew-summary-touch-file
 				      (mew-expand-folder dir)))
@@ -959,7 +1018,8 @@ If called with '\\[universal-argument]', re-retrieve messages in the region."
 	(when (and fld msg (null part))
 	  (save-excursion
 	    (beginning-of-line)
-	    (when (looking-at (mew-shimbun-unseen-regex))
+	    (when (looking-at (or mew-shimbun-unseen-regex
+				  (mew-shimbun-unseen-regex)))
 	      ;; in normal or thread folder
 	      (mew-mark-unmark)
 	      (set-buffer-modified-p nil)
